/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER, Ghostar
 * Version: 1.0
 */

using System;
using System.Collections;
using System.Data;
using System.Globalization;
using TopCoder.Data.ConnectionFactory;
using TopCoder.Document.Index.WordStreams;
using TopCoder.Util.ConfigurationManager;

namespace TopCoder.Document.Index.Persistence.DB
{
    /// <summary>
    /// This class implements the <see cref="IDocumentIndexSource"/> in a database related manner. The saving and
    /// loading is all done to a database, based on connection parameters given. A transaction is used to queue up
    /// changes and then commit them all at once.
    /// </summary>
    /// <remarks>
    /// This class is mutable and is not thread safe.
    /// </remarks>
    public class DBDocumentIndexSource : IDocumentIndexSource, IComparer, IDisposable
    {
        /// <summary>
        /// The configuration manager namespace used in the zero parameter constructor. The individual parameters are
        /// described in the component specification.
        /// </summary>
        public const string DefaultNamespace = "TopCoder.Document.Index.Persistence.DB";

        /// <summary>
        /// This variable holds the configuration information for this class, including the name of the connection
        /// loaded, as well as the document ID. This value is initialized to a new
        /// <see cref="DBDocumentIndexParameters"/> instance in the constructor, using values read from the namespace
        /// given to the constructor.
        /// </summary>
        private DBDocumentIndexParameters parameters;

        /// <summary>
        /// This property returns a newly created <see cref="IndexInfo"/> instance, created with the assembly and file
        /// name of this class, along with a <see cref="DBDocumentIndexParameters"/> instance that holds the path to
        /// the file that the current instance is currently working against. The
        /// <see cref="DBDocumentIndexParameters"/> should be created with the connection type name, connection string,
        /// auto commit, and document ID properties of this class.
        /// </summary>
        /// <value>
        /// An <see cref="IndexInfo"/> instance, populated with relevant data.
        /// </value>
        public IndexInfo IndexInfo
        {
            get
            {
                // Just create an instance. Note the fully qualified assembly name.
                return new IndexInfo(GetType().Assembly.FullName, GetType().FullName,
                    parameters);
            }
        }

        /// <summary>
        /// This property provides read and write access to the culture member variable of this class, used to set the
        /// culture to use for string comparisons.
        /// </summary>
        /// <exception cref="ArgumentNullException">If <paramref name="value"/> is null.</exception>
        /// <exception cref="InvalidOperationException">If the options are changed for a non-empty index. This is
        /// dangerous, since changes of sorting order could invalidate existing data.</exception>
        /// <exception cref="SaveIndexDataException">Any SQL error while setting.</exception>
        /// <exception cref="LoadIndexDataException">Any SQL error while getting.</exception>
        /// <value>
        /// The default value, if nothing else is specified, is en-US.
        /// </value>
        public CultureInfo Culture
        {
            get
            {
                ReadCompareSettingsNow();
                return culture;
            }
            set
            {
                if (value == null)
                {
                    throw new ArgumentNullException("value");
                }
                if (words.Count > 0)
                {
                    throw new
                        InvalidOperationException("Sorting order can only be changed when no words are present.");
                }
                culture = value;
                WriteCompareSettingsNow();
            }
        }

        /// <summary>
        /// This property holds compare options.  It controls, together with the <see cref="Culture"/> property, how
        /// string comparisons are performed.
        /// </summary>
        /// <exception cref="InvalidOperationException">If the options are changed for a non-empty index. This is
        /// dangerous, since changes of sorting order could invalidate existing data.</exception>
        /// <exception cref="SaveIndexDataException">Any SQL error while setting.</exception>
        /// <exception cref="LoadIndexDataException">Any SQL error while getting.</exception>
        /// <value>
        /// The default value is <see cref="System.Globalization.CompareOptions.None"/>, but any value can be used.
        /// </value>
        public CompareOptions CompareOptions
        {
            get
            {
                ReadCompareSettingsNow();
                return compareOptions;
            }
            set
            {
                if (words.Count > 0)
                {
                    throw new
                        InvalidOperationException("Sorting order can only be changed when no words are present.");
                }
                compareOptions = value;
                WriteCompareSettingsNow();
            }
        }

        /// <summary>
        /// This field stores the parameters for which compare options were last read.
        /// </summary>
        private IParameters lastReadCompareFrom = null;

        /// <summary>
        /// Reads the compare settings; culture and compare options from the database, unless they have already been
        /// read for this connection.
        /// </summary>
        /// <exception cref="LoadIndexDataException">Any SQL error.</exception>
        private void ReadCompareSettingsNow()
        {
            // Needs to (re)read?
            if (parameters != lastReadCompareFrom)
            {
                try
                {
                    // The document ID is set in the property
                    using (IDataReader reader = ReadCompareSettings.ExecuteReader())
                    {
                        reader.Read();
                        culture = new CultureInfo(((string) reader[0]).Trim());
                        compareOptions = (CompareOptions) Enum.Parse(typeof(CompareOptions), (string) reader[1]);
                    }
                }
                catch (Exception e)
                {
                    throw new LoadIndexDataException(parameters, e, "Error reading comparison settings.");
                }
                lastReadCompareFrom = parameters;
            }
        }

        /// <summary>
        /// Writes the compare settings; culture and compare options to the database. The existing transaction is used.
        /// </summary>
        /// <exception cref="SaveIndexDataException">Any SQL error.</exception>
        private void WriteCompareSettingsNow()
        {
            try
            {
                ((IDataParameter) WriteCompareSettings.Parameters[0]).Value = culture.ToString();
                ((IDataParameter) WriteCompareSettings.Parameters[1]).Value = compareOptions.ToString();
                // The document ID is set in the property

                WriteCompareSettings.ExecuteNonQuery();
            }
            catch (Exception e)
            {
                throw new SaveIndexDataException(e, "Error writing comparison settings.");
            }
        }

        /// <summary>
        /// This private member variable holds the culture to use when comparing two strings. This allows us to
        /// provide support for multiple languages and alphabets when searching through a given document. This member
        /// can be changed through the <see cref="Culture"/> property.
        /// </summary>
        private CultureInfo culture;

        /// <summary>
        /// This member variable holds options, beyond the culture, for comparing strings in this particular class.
        /// This value is initialized in the constructor, and it can be accessed using the <see cref="CompareOptions"/>
        /// property.
        /// </summary>
        private CompareOptions compareOptions;

        /// <summary>
        /// This member variable holds the loaded words, loaded from the database that this instance is working
        /// against. The keys in this dictionary are the words, and the values are <see cref="IList"/> instances of
        /// integer locations in the document where this word can be found. This collection is loaded by the
        /// <see cref="Load"/> method and accessed by the indexers in this class.
        /// </summary>
        private SortedList words;

        /// <summary>
        /// This member variable holds a list of all word IDs that indicate duplicates. Those can be created if the
        /// comparison settings create identity for different words. The keys are word strings, the values are lists.
        /// Entries are only created for those words where duplicates actually exist.
        /// </summary>
        private SortedList alternateIDs;

        /// <summary>
        /// This connection is the actual connection that is used to talk to the database for updates. This value is
        /// set using the Connection Factory component.
        /// </summary>
        private IDbConnection connection = null;

        /// <summary>
        /// A lazy-initialized property for the current connection.
        /// </summary>
        /// <exception cref="Exception">Any exception may be thrown. Consumers need to wrap them.</exception>
        /// <value>
        /// An <see cref="IDbConnection"/>, never null.
        /// </value>
        private IDbConnection Connection
        {
            get
            {
                if (connection == null)
                {
                    // Ask the connection factory
                    connection = ConnectionManager.Instance.CreatePredefinedDbConnection(parameters.ConnectionName);
                    connection.Open();

                    // Start a transaction
                    if (transaction != null)
                    {
                        // Close it, but never fail
                        try
                        {
                            transaction.Dispose();
                        }
                        catch
                        {
                        }
                    }

                    transaction = connection.BeginTransaction();
                }

                return connection;
            }
        }

        /// <summary>
        /// All updates are added to this transaction, and they are committed in the <see cref="Save"/> method. This is
        /// initialized to a new <see cref="IDbTransaction"/> implementation using
        /// <see cref="IDbConnection.BeginTransaction"/>.
        /// </summary>
        private IDbTransaction transaction = null;

        /// <summary>
        /// The command used to load all locations for all words in this document.
        /// </summary>
        private IDbCommand loadWords;

        /// <summary>
        /// The command used to load all locations for all words in this document. Managed to always point to the
        /// current connection and transaction.
        /// </summary>
        /// <value>
        /// <see cref="loadWords"/>
        /// </value>
        private IDbCommand LoadWords
        {
            get
            {
                if (loadWords == null || loadWords.Connection != Connection)
                {
                    if (loadWords != null)
                    {
                        // Close, but never fail
                        try
                        {
                            loadWords.Dispose();
                        }
                        catch (Exception)
                        {
                        }
                    }

                    loadWords = Connection.CreateCommand();
                    loadWords.CommandText = "SELECT Word.word_id, word, location FROM Word JOIN Location ON " +
                        "Location.word_id=Word.word_id WHERE document_id=? ORDER BY word ASC, word_id ASC";

                    IDbDataParameter document_id = loadWords.CreateParameter();
                    document_id.Value = parameters.DocumentID;
                    loadWords.Parameters.Add(document_id);

                    // Preparing is often unneeded in modern systems, but it should not hurt, we're doing this once
                    // per connection only.
                    loadWords.Prepare();
                }

                if (loadWords.Transaction != transaction)
                {
                    loadWords.Transaction = transaction;
                }

                return loadWords;
            }
        }

        /// <summary>
        /// The command used to add a location for a word.
        /// </summary>
        private IDbCommand addLocation;

        /// <summary>
        /// The command used to add a location for a word. Managed to always point to the current connection and
        /// transaction.
        /// </summary>
        /// <value>
        /// <see cref="addLocation"/>
        /// </value>
        private IDbCommand AddLocation
        {
            get
            {
                if (addLocation == null || addLocation.Connection != Connection)
                {
                    if (addLocation != null)
                    {
                        // Close, but never fail
                        try
                        {
                            addLocation.Dispose();
                        }
                        catch (Exception)
                        {
                        }
                    }

                    addLocation = Connection.CreateCommand();
                    addLocation.CommandText = "INSERT INTO Location (word_id, location, document_id) VALUES " +
                        "(?, ?, ?)";

                    IDbDataParameter word_id = addLocation.CreateParameter();
                    addLocation.Parameters.Add(word_id);

                    IDbDataParameter location = addLocation.CreateParameter();
                    addLocation.Parameters.Add(location);

                    IDbDataParameter document_id = addLocation.CreateParameter();
                    document_id.Value = parameters.DocumentID;
                    addLocation.Parameters.Add(document_id);

                    // Preparing is often unneeded in modern systems, but it should not hurt, we're doing this once
                    // per connection only.
                    addLocation.Prepare();
                }

                if (addLocation.Transaction != transaction)
                {
                    addLocation.Transaction = transaction;
                }

                return addLocation;
            }
        }

        /// <summary>
        /// The command used to check if a word exists.
        /// </summary>
        private IDbCommand checkWord;

        /// <summary>
        /// The command used to check if a word exists. Managed to always point to the current connection and
        /// transaction.
        /// </summary>
        /// <value>
        /// <see cref="checkWord"/>
        /// </value>
        private IDbCommand CheckWord
        {
            get
            {
                if (checkWord == null || checkWord.Connection != Connection)
                {
                    if (checkWord != null)
                    {
                        // Close, but never fail
                        try
                        {
                            checkWord.Dispose();
                        }
                        catch (Exception)
                        {
                        }
                    }

                    checkWord = Connection.CreateCommand();
                    checkWord.CommandText = "SELECT word_id FROM Word WHERE word=?";

                    IDbDataParameter word = checkWord.CreateParameter();
                    checkWord.Parameters.Add(word);

                    // Preparing is often unneeded in modern systems, but it should not hurt, we're doing this once
                    // per connection only.
                    checkWord.Prepare();
                }

                if (checkWord.Transaction != transaction)
                {
                    checkWord.Transaction = transaction;
                }

                return checkWord;
            }
        }

        /// <summary>
        /// The command used to insert a new word.
        /// </summary>
        private IDbCommand insertWord;

        /// <summary>
        /// The command used to insert a new word. Managed to always point to the current connection and transaction.
        /// </summary>
        /// <value>
        /// <see cref="insertWord"/>
        /// </value>
        private IDbCommand InsertWord
        {
            get
            {
                if (insertWord == null || insertWord.Connection != Connection)
                {
                    if (insertWord != null)
                    {
                        // Close, but never fail
                        try
                        {
                            insertWord.Dispose();
                        }
                        catch (Exception)
                        {
                        }
                    }

                    insertWord = Connection.CreateCommand();
                    insertWord.CommandText = "INSERT INTO Word (word) values (?)";

                    IDbDataParameter word = insertWord.CreateParameter();
                    insertWord.Parameters.Add(word);

                    // Preparing is often unneeded in modern systems, but it should not hurt, we're doing this once
                    // per connection only.
                    insertWord.Prepare();
                }

                if (insertWord.Transaction != transaction)
                {
                    insertWord.Transaction = transaction;
                }

                return insertWord;
            }
        }

        /// <summary>
        /// The command used to remove a word.
        /// </summary>
        private IDbCommand removeWord;

        /// <summary>
        /// The command used to remove a word. Managed to always point to the current connection and transaction.
        /// </summary>
        /// <value>
        /// <see cref="removeWord"/>
        /// </value>
        private IDbCommand RemoveWord
        {
            get
            {
                if (removeWord == null || removeWord.Connection != Connection)
                {
                    if (removeWord != null)
                    {
                        // Close, but never fail
                        try
                        {
                            removeWord.Dispose();
                        }
                        catch (Exception)
                        {
                        }
                    }

                    removeWord = Connection.CreateCommand();
                    removeWord.CommandText = "DELETE FROM Location WHERE word_id=? AND document_id=?";

                    IDbDataParameter word_id = removeWord.CreateParameter();
                    removeWord.Parameters.Add(word_id);

                    IDbDataParameter document_id = removeWord.CreateParameter();
                    document_id.Value = parameters.DocumentID;
                    removeWord.Parameters.Add(document_id);

                    // Preparing is often unneeded in modern systems, but it should not hurt, we're doing this once
                    // per connection only.
                    removeWord.Prepare();
                }

                if (removeWord.Transaction != transaction)
                {
                    removeWord.Transaction = transaction;
                }

                return removeWord;
            }
        }

        /// <summary>
        /// The command used to read culture and compare options.
        /// </summary>
        private IDbCommand readCompareSettings;

        /// <summary>
        /// The command used to read culture and compare options. Managed to always point to the current connection and
        /// transaction.
        /// </summary>
        /// <value>
        /// <see cref="readCompareSettings"/>
        /// </value>
        private IDbCommand ReadCompareSettings
        {
            get
            {
                if (readCompareSettings == null || readCompareSettings.Connection != Connection)
                {
                    if (readCompareSettings != null)
                    {
                        // Close, but never fail
                        try
                        {
                            readCompareSettings.Dispose();
                        }
                        catch (Exception)
                        {
                        }
                    }

                    readCompareSettings = Connection.CreateCommand();
                    readCompareSettings.CommandText =
                        "SELECT culture, compare_options FROM Document WHERE document_id=?";

                    IDbDataParameter document_id = readCompareSettings.CreateParameter();
                    document_id.Value = parameters.DocumentID;
                    readCompareSettings.Parameters.Add(document_id);

                    // Reading compare settings is very infrequent. There is no need to Prepare this command.
                }

                if (readCompareSettings.Transaction != transaction)
                {
                    readCompareSettings.Transaction = transaction;
                }

                return readCompareSettings;
            }
        }

        /// <summary>
        /// The command used to write culture and compare options.
        /// </summary>
        private IDbCommand writeCompareSettings;

        /// <summary>
        /// The command used to write culture and compare options. Managed to always point to the current connection
        /// and transaction.
        /// </summary>
        /// <value>
        /// <see cref="writeCompareSettings"/>
        /// </value>
        private IDbCommand WriteCompareSettings
        {
            get
            {
                if (writeCompareSettings == null || writeCompareSettings.Connection != Connection)
                {
                    if (writeCompareSettings != null)
                    {
                        // Close, but never fail
                        try
                        {
                            writeCompareSettings.Dispose();
                        }
                        catch (Exception)
                        {
                        }
                    }

                    writeCompareSettings = Connection.CreateCommand();
                    writeCompareSettings.CommandText =
                        "UPDATE Document SET culture=?, compare_options=? WHERE document_id=?";

                    IDbDataParameter culture = writeCompareSettings.CreateParameter();
                    writeCompareSettings.Parameters.Add(culture);

                    IDbDataParameter compareOptions = writeCompareSettings.CreateParameter();
                    writeCompareSettings.Parameters.Add(compareOptions);

                    IDbDataParameter document_id = writeCompareSettings.CreateParameter();
                    document_id.Value = parameters.DocumentID;
                    writeCompareSettings.Parameters.Add(document_id);

                    // Preparing is often unneeded in modern systems, but it should not hurt, we're doing this once
                    // per connection only.
                    writeCompareSettings.Prepare();
                }

                if (writeCompareSettings.Transaction != transaction)
                {
                    writeCompareSettings.Transaction = transaction;
                }

                return writeCompareSettings;
            }
        }

        /// <summary>
        /// The command used to read culture and compare options.
        /// </summary>
        private IDbCommand clearCommand;

        /// <summary>
        /// The command used to clear all stored locations for a specific document. Managed to always point to the
        /// current connection and transaction.
        /// </summary>
        /// <value>
        /// <see cref="clearCommand"/>
        /// </value>
        private IDbCommand ClearCommand
        {
            get
            {
                if (clearCommand == null || clearCommand.Connection != Connection)
                {
                    if (clearCommand != null)
                    {
                        // Close, but never fail
                        try
                        {
                            clearCommand.Dispose();
                        }
                        catch (Exception)
                        {
                        }
                    }

                    clearCommand = Connection.CreateCommand();
                    clearCommand.CommandText =
                        "DELETE FROM Location WHERE document_id=?";

                    IDbDataParameter document_id = clearCommand.CreateParameter();
                    document_id.Value = parameters.DocumentID;
                    clearCommand.Parameters.Add(document_id);

                    // Preparing is often unneeded in modern systems, but it should not hurt, we're doing this once
                    // per connection only.
                    clearCommand.Prepare();
                }

                if (clearCommand.Transaction != transaction)
                {
                    clearCommand.Transaction = transaction;
                }

                return clearCommand;
            }
        }

        /// <summary>
        /// This constructor creates a new <see cref="DBDocumentIndexSource"/> based on configuration parameters read
        /// from the namespace given. The parameters read should include the connection name, and the ID of the
        /// document in the database this class will work against.
        /// </summary>
        /// <param name="nameSpace">The filepath to use when loading.</param>
        /// <exception cref="ArgumentException">If <paramref name="nameSpace"/> is empty.</exception>
        /// <exception cref="ArgumentNullException">If <paramref name="nameSpace"/> is null.</exception>
        /// <exception cref="DBPersistenceConfigurationException">Any namespace or parameter missing.</exception>
        public DBDocumentIndexSource(string nameSpace)
        {
            if (nameSpace == null)
            {
                throw new ArgumentNullException("nameSpace");
            }
            if (nameSpace.Trim().Length == 0)
            {
                throw new ArgumentException("Empty string specified.", "nameSpace");
            }

            words = new SortedList(this);
            alternateIDs = new SortedList(this);

            try
            {
                ConfigManager cm = ConfigManager.GetInstance();
                parameters = new DBDocumentIndexParameters(
                    cm.GetValue(nameSpace, DBPersistenceParameters.CONNECTIONNAME_PROPERTY).Trim(),
                    long.Parse(cm.GetValue(nameSpace, DBDocumentIndexParameters.DOCUMENTID_PROPERTY)));
            }
            catch (Exception e)
            {
                throw new DBPersistenceConfigurationException("Configuration error.", e);
            }
        }

        /// <summary>
        /// Default constructor. A default namespace is used. No loading is performed.
        /// </summary>
        public DBDocumentIndexSource()
            : this(DefaultNamespace)
        {
        }

        /// <summary>
        /// This method adds the given word to the XML document. This method should just update the internally stored
        /// words list, creating a new <see cref="ArrayList"/>.
        /// </summary>
        /// <param name="wordInfo">The word info to add.</param>
        /// <exception cref="ArgumentNullException">If <paramref name="wordInfo"/> is null.</exception>
        /// <exception cref="ArgumentException">If the word in <paramref name="wordInfo"/> is empty.</exception>
        /// <exception cref="SaveIndexDataException">Any SQL error.</exception>
        public void Add(WordInfo wordInfo)
        {
            if (wordInfo == null)
            {
                throw new ArgumentNullException("wordInfo");
            }
            string word = wordInfo.Word.Trim();
            if (word.Length == 0)
            {
                throw new ArgumentException("Empty word string specified.", "wordInfo");
            }

            IList list = words[word] as IList;

            if (list == null)
            {
                // Create a new list. Let's assume that it's very frequent that words only occur once (that's quite
                // true for most data in real corpora), then it will grow just like normal.
                list = new ArrayList(1);
                words[word] = list;

                // Now, we can add the new position to the list, no matter how it got there.
            }

            list.Add(wordInfo.Position);
            AddToID(GetWordID(word), wordInfo.Position);
        }

        /// <summary>
        /// This method removes the given word and its locations from the words collection.
        /// </summary>
        /// <param name="word">The word to remove.</param>
        /// <exception cref="ArgumentNullException">If <paramref name="word"/> is null.</exception>
        /// <exception cref="ArgumentException">If <paramref name="word"/> is an empty string.</exception>
        /// <exception cref="SaveIndexDataException">Any SQL error.</exception>
        public void Remove(string word)
        {
            if (word == null)
            {
                throw new ArgumentNullException("word");
            }
            word = word.Trim();
            if (word.Length == 0)
            {
                throw new ArgumentException("Empty string specified.", "word");
            }

            if (words.Contains(word))
            {
                // Get back to the specific word form used in the database.
                RemoveID(GetWordID(word));
                IList list = alternateIDs[word] as IList;
                foreach (object id in list)
                {
                    RemoveID(id);
                }

                words.Remove(word);
            }
        }

        /// <summary>
        /// This method returns whether or not the document represented by this class contains the given word.
        /// </summary>
        /// <param name="word">The word to check for.</param>
        /// <exception cref="ArgumentNullException">If <paramref name="word"/> is null.</exception>
        /// <exception cref="ArgumentException">If <paramref name="word"/> is an empty string.</exception>
        /// <returns>True if it exists, false if not.</returns>
        public bool Contains(string word)
        {
            if (word == null)
            {
                throw new ArgumentNullException("word");
            }
            word = word.Trim();
            if (word.Length == 0)
            {
                throw new ArgumentException("Empty string specified.", "word");
            }

            IList list = words[word] as IList;

            // It doesn't exist
            if (list == null)
            {
                return false;
            }

            // If there is such a list, and it's non-empty, return true.
            return list.Count > 0;
        }

        /// <summary>
        /// This method clears all words from the document.
        /// </summary>
        /// <exception cref="SaveIndexDataException">Any SQL error.</exception>
        public void Clear()
        {
            try
            {
                ClearCommand.ExecuteNonQuery();
            }
            catch (Exception e)
            {
                throw new SaveIndexDataException(e, "Error clearing.");
            }
            words.Clear();
        }

        /// <summary>
        /// This method just returns the enumerator for the words collection, <see cref="IDictionary.GetEnumerator"/>.
        /// This method can be used by the user to enumerate through all words in a given document index.
        /// </summary>
        /// <returns>
        /// An enumerator instance.
        /// </returns>
        public IDictionaryEnumerator GetEnumerator()
        {
            return words.GetEnumerator();
        }

        /// <summary>
        /// This method loads the data from the DB store according to the schema described by the ERD and SQL files in
        /// the docs folder.
        /// </summary>
        /// <remarks>
        /// <para>
        /// The component will request the database words in a monotonically increasing order (ORDER BY ASC). The total
        /// loading time should be O(N log N). This is quite acceptable. If the database collation differs from the
        /// comparison settings used by the component, one can reach a worst case of O(N^2). The O(N^2) could be
        /// eliminated if we accepted doing an additional copy. Doing that in the general case
        /// would however possibly be quite expensive, especially when handling huge data sets.
        /// </para>
        /// <para>
        /// Please note that the state will be undefined if the load operation fails.
        /// </para>
        /// </remarks>
        /// <exception cref="ArgumentNullException">If <paramref name="parameters"/> is null.</exception>
        /// <exception cref="ArgumentException">If <paramref name="parameters"/> is not a
        /// <see cref="DBDocumentIndexParameters"/> instance.
        /// </exception>
        /// <exception cref="LoadIndexDataException">Any error while loading the words.</exception>
        /// <param name="parameters">The parameters specifying the DB to load from.</param>
        public void Load(IParameters parameters)
        {
            if (parameters == null)
            {
                throw new ArgumentNullException("parameters");
            }
            if (!(parameters is DBDocumentIndexParameters))
            {
                throw new ArgumentException("Specified parameters not of DBDocumentIndexParameters type.",
                    "parameters");
            }

            this.parameters = (DBDocumentIndexParameters) parameters;

            // Our Dispose will just close the current connection, but the object will remain usable.
            Dispose();
            // A new connection will be created, based on the current parameters.
            try
            {
                // Needed, since we can't open another reader during the real operations
                words.Clear();
                ReadCompareSettingsNow();

                using (IDataReader reader = LoadWords.ExecuteReader())
                {
                    while (reader.Read())
                    {
                        object word_id = reader[0];
                        // Trim needed especially if CHAR is used (not VARCHAR)
                        string word = ((string) reader[1]).Trim();
                        // The type is defined as long, but the schema says Integer, which results in Int32 with
                        // most settings
                        long location = Convert.ToInt64(reader[2]);

                        IList list;
                        int index = words.IndexOfKey(word);

                        // Does this word already exist or not?
                        if (index == -1)
                        {
                            list = new ArrayList(1);
                            list.Add(location);

                            words.Add(word, list);
                        }
                        else
                        {
                            list = (IList) words.GetByIndex(index);
                            list.Add(location);

                            // Add an alternate, if the literal word strings are different
                            if (word != (string) words.GetKey(index))
                            {
                                index = alternateIDs.IndexOfKey(word);

                                if (index == -1)
                                {
                                    // New alternate list
                                    list = new ArrayList(1);
                                    list.Add(word_id);

                                    alternateIDs[word] = list;
                                }
                                else
                                {
                                    list = (IList) alternateIDs.GetByIndex(index);

                                    // We requested the word list in sorted order, so we can avoid duplicates by just
                                    // verifying that the previous word was different from this one.
                                    if (list[list.Count - 1].Equals(word_id))
                                    {
                                        list.Add(word_id);
                                    }
                                }
                            }
                        }
                    }
                }
            }
            catch (Exception e)
            {
                // Avoid simulating that a Load actually took place, when it did not.
                // This will create an undefined state, but it can be solved by a following, valid, Load.
                parameters = null;
                words.Clear();
                Dispose();

                throw new LoadIndexDataException(parameters, e);
            }
            finally
            {
                // Try to commit the current transaction, by using Save. If we didn't do this, the current transaction
                // would always include the huge Load operation, which could easily block other transactions, depending
                // on the concurrency algorithm and lock granularity.
                //
                // However, if this fails, it's not a catastrophic failure in itself. We're just trying to be kind.
                try
                {
                    Save();
                }
                catch (Exception)
                {
                }
            }
        }

        /// <summary>
        /// This method commits any changes made in the current transaction. Note that updates in other indexes may
        /// fail if they would affect data that's already been affected by this transaction.
        /// </summary>
        /// <exception cref="SaveIndexDataException">Any error while saving.</exception>
        public void Save()
        {
            try
            {
                // Nope, saving something that hasn't started won't do much good
                if (connection != null)
                {
                    if (transaction != null)
                    {
                        transaction.Commit();
                        transaction.Dispose();
                    }
                    transaction = Connection.BeginTransaction();
                }
            }
            catch (Exception e)
            {
                throw new SaveIndexDataException(e, "Error during transaction commit.");
            }
        }

        /// <summary>
        /// This method is implemented to compare two strings using the culture info contained in this class. This
        /// method is used by the internal data store when sorting the words in the list. We also use the
        /// <see cref="CompareOptions"/> property here.
        /// </summary>
        /// <param name="x">One string to compare against <paramref name="y"/>.</param>
        /// <param name="y">One string to compare against <paramref name="x"/>.</param>
        /// <exception cref="ArgumentException">If any argument is not a string (null values are allowed).</exception>
        /// <returns>Negative if x &lt; y, positive if y &gt; x, zero if equal.</returns>
        public int Compare(object x, object y)
        {
            string xStr = x as string;
            string yStr = y as string;

            // Compare is called numerous times while handling the SortedList. The parameters are almost never
            // non-strings or null, and this is an ever so slight optimization for that fact. (It helps real run time
            // a tiny bit, at least on P6-based CPUs.)
            if (xStr == null || yStr == null)
            {
                if (xStr == null && x != null)
                {
                    throw new ArgumentException("Not a string.", "x");
                }
                if (yStr == null && y != null)
                {
                    throw new ArgumentException("Not a string.", "y");
                }
            }

            return Culture.CompareInfo.Compare(xStr, yStr, CompareOptions);
        }

        /// <summary>
        /// Gets/sets the <see cref="IList"/> of integer positions for a specific word string.
        /// </summary>
        /// <param name="word">The word to change/read the list for.</param>
        /// <exception cref="ArgumentException">If <paramref name="word"/> is empty.</exception>
        /// <exception cref="ArgumentNullException">If <paramref name="word"/> is null.</exception>
        /// <exception cref="ArgumentNullException">If <paramref name="value"/> is null (only setter).</exception>
        /// <exception cref="ArgumentException">If any element in <paramref name="value"/> is null/impossible to cast
        /// into a <see cref="long"/>.
        /// </exception>
        /// <exception cref="SaveIndexDataException">Any SQL error while updating the database representation.
        /// </exception>
        /// <value>
        /// An <see cref="IList"/> of <see cref="long"/> values.
        /// </value>
        public IList this[string word]
        {
            get
            {
                if (word == null)
                {
                    throw new ArgumentNullException("word");
                }
                word = word.Trim();
                if (word.Length == 0)
                {
                    throw new ArgumentException("Empty string specified.", "word");
                }

                IList original = (IList) words[word];

                if (original == null)
                {
                    return null;
                }

                return new ArrayList(original);
            }
            set
            {
                if (value == null)
                {
                    throw new ArgumentNullException("value");
                }
                if (word == null)
                {
                    throw new ArgumentNullException("word");
                }
                word = word.Trim();
                if (word.Length == 0)
                {
                    throw new ArgumentException("Empty string specified.", "word");
                }

                IList newList = new ArrayList(value.Count);
                try
                {
                    // This cast will fail, even for null values, as Int64 is not a reference type.
                    foreach (object element in value)
                    {
                        if (element == null)
                        {
                            throw new ArgumentException("Null values not allowed.", "value");
                        }
                        newList.Add(Convert.ToInt64(element));
                    }
                }
                catch (InvalidCastException e)
                {
                    throw new ArgumentException("Only long values allowed.", "value", e);
                }

                // Update the database, remove, add, finish
                object wordID = GetWordID(word);
                Remove(word);
                foreach (long location in newList)
                {
                    AddToID(wordID, location);
                }

                words[word] = newList;
            }
        }

        /// <summary>
        /// Gets the word ID to use when adding to a specific word. The word is inserted if it doesn't already exist.
        /// </summary>
        /// <param name="word">This word.</param>
        /// <exception cref="SaveIndexDataException">Any SQL error while updating the database representation.
        /// </exception>
        /// <returns>The database ID for that word.</returns>
        private object GetWordID(string word)
        {
            try
            {
                int key = words.IndexOfKey(word);

                // Normalize to the form we know, if we know any
                // for example, if the DB stores entr and we get a request for entre
                if (key != -1)
                {
                    word = (string) words.GetKey(key);
                }

                for (int i = 0; i < 2; i++)
                {
                    ((IDbDataParameter) CheckWord.Parameters[0]).Value = word;
                    using (IDataReader reader = CheckWord.ExecuteReader())
                    {
                        if (reader.Read())
                        {
                            // Return the first column
                            return reader[0];
                        }
                    }

                    ((IDbDataParameter) InsertWord.Parameters[0]).Value = word;
                    InsertWord.ExecuteNonQuery();
                }
            }
            catch (Exception e)
            {
                throw new SaveIndexDataException(e, "Error identifying word ID.");
            }

            throw new SaveIndexDataException(null, "Unable to insert/locate word ID.");
        }

        /// <summary>
        /// Adds a specific location to a specific word (in this document).
        /// </summary>
        /// <param name="word_id">The word ID to add to.</param>
        /// <param name="location">The location value to add.</param>
        /// <exception cref="SaveIndexDataException">Any SQL error while updating the database representation.
        /// </exception>
        private void AddToID(object word_id, long location)
        {
            try
            {
                ((IDbDataParameter) AddLocation.Parameters[0]).Value = word_id;
                ((IDbDataParameter) AddLocation.Parameters[1]).Value = location;

                AddLocation.ExecuteNonQuery();
            }
            catch (Exception e)
            {
                throw new SaveIndexDataException(e, "Error adding to word.");
            }
        }

        /// <summary>
        /// Removes all the locations for a specific word (ID) in the current document.
        /// </summary>
        /// <param name="word_id">The word ID to remove the locations for.</param>
        /// <exception cref="SaveIndexDataException">Any SQL error while updating the database representation.
        /// </exception>
        private void RemoveID(object word_id)
        {
            try
            {
                ((IDbDataParameter) RemoveWord.Parameters[0]).Value = word_id;

                RemoveWord.ExecuteNonQuery();
            }
            catch (Exception e)
            {
                throw new SaveIndexDataException(e, "Error identifying word ID.");
            }
        }

        /// <summary>
        /// Disposes the current connection. The active transaction is rolled back. However, except for this, the
        /// database connection will be resurrected when loading again. Note that any changes to the local
        /// representation are NOT rolled back, so disposing an object with active changes will create a
        /// synchronization mismatch.
        /// </summary>
        /// <remarks>
        /// We do only encapsulate other managed resources, so a separate finalizer wouldn't do any good.
        /// </remarks>
        public void Dispose()
        {
            try
            {
                if (connection != null)
                {
                    if (transaction != null)
                    {
                        transaction.Rollback();
                        transaction.Dispose();
                    }
                    connection.Dispose();
                    connection = null;
                    transaction = null;
                }
            }
            catch (Exception)
            {
            }
        }
    }
}
