/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER, Ghostar
 * Version: 1.0
 */

using NUnit.Framework;
using System;
using System.Data;
using TopCoder.Data.ConnectionFactory;
using TopCoder.Util.ConfigurationManager;

namespace TopCoder.Document.Index.Persistence.DB
{
    /// <summary>
    /// This class provides common tests for database tests. Any error in setting up the fixture probably indicates a
    /// misconfiguration of the database connection or configuration manager namespaces. The test case will populate
    /// some test data matching the XML test files in the database, but it requires the proper schema to already be in
    /// place.
    /// </summary>
    [TestFixture]
    public abstract class DBTests
    {
        /// <summary>
        /// A list of commands that are executed when setting up the test.
        /// </summary>
        private static readonly string[] commands = new string[]
            {
                "DELETE FROM Location",
                "DELETE FROM Word",
                "DELETE FROM Document",
                "INSERT INTO Document (document_id, document_name, compare_options, culture) VALUES" +
                "(1, 'index', 'IgnoreNonSpace', 'fr-FR')",
                "INSERT INTO Document (document_id, document_name, compare_options, culture) VALUES" +
                "(2, 'index2', 'None', 'en-US')",
                "INSERT INTO Word (word_id, word) VALUES (1, 'entr')",
                "INSERT INTO Word (word_id, word) VALUES (3, 'entre')",
                "INSERT INTO Word (word_id, word) VALUES (2, 'rouge')",
                "INSERT INTO Location (word_id, document_id, location) VALUES (1, 1, 3)",
                "INSERT INTO Location (word_id, document_id, location) VALUES (3, 1, 1)",
                "INSERT INTO Location (word_id, document_id, location) VALUES (2, 1, 3)"
            };

        /// <summary>
        /// Sets up the testing fixture. This includes setting up the database connection to add some test data.
        /// However, the proper schema must already be in place.
        /// </summary>
        [SetUp]
        public virtual void Setup()
        {
            try
            {
                ConfigManager.GetInstance().Clear(false);
                ConfigManager.GetInstance().LoadFile(@"..\..\test_files\config.xml");
                RunCommands(0, commands.Length);
            }
            catch (Exception)
            {
                Console.Error.WriteLine("\nError initializing test fixture for database tests. This probably");
                Console.Error.WriteLine("indicates a configuration problem.\n");

                throw;
            }
        }

        /// <summary>
        /// Tears down the test fixture, resets the database.
        /// </summary>
        [TearDown]
        public virtual void Teardown()
        {
            // The first three commands truncate
            RunCommands(0, 3);
            ConfigManager.GetInstance().Clear(true);
        }

        /// <summary>
        /// Runs commands from the <see cref="commands"/> array.
        /// </summary>
        /// <param name="start">The start index.</param>
        /// <param name="count">The number of commands to run.</param>
        private static void RunCommands(int start, int count)
        {
            using (IDbConnection connection = ConnectionManager.Instance.CreatePredefinedDbConnection("Test"))
            {
                connection.Open();
                using (IDbCommand command = connection.CreateCommand())
                {
                    for (int i = start; i < start + count; i++)
                    {
                        command.CommandText = commands[i];
                        command.ExecuteNonQuery();
                    }
                }
            }
        }
    }
}
