/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER
 * Version: 1.0
 */

using NUnit.Framework;
using System;
using System.Collections;
using System.Globalization;
using System.IO;
using TopCoder.Document.Index.WordStreams;

namespace TopCoder.Document.Index.Persistence.Xml
{
    /// <summary>
    /// Tests the <see cref="XmlDocumentIndexSource"/> class. Most of the complex logic in that class relates to
    /// loading and saving XML representations.
    /// </summary>
    /// <remarks>
    /// This class "implements" <see cref="IParameters"/> to supply itself as an (incorrect) parameter object.
    /// </remarks>
    [TestFixture]
    public class TestXmlDocumentIndexSource : IParameters
    {
        /// <summary>
        /// A test object used in several tests, created based on a file.
        /// </summary>
        private XmlDocumentIndexSource testObject1;

        /// <summary>
        /// A test object crated with no file.
        /// </summary>
        private XmlDocumentIndexSource testObject2;

        /// <summary>
        /// A reference to the type used, for testing.
        /// </summary>
        private static readonly Type type = typeof(XmlDocumentIndexSource);

        /// <summary>
        /// The name of the test file.
        /// </summary>
        private const string TEST_FILENAME = @"..\..\test_files\testarchive.xml";

        /// <summary>
        /// The name of the test file with all in the main file.
        /// </summary>
        private const string TEST_MAINFILENAME = @"..\..\test_files\testmainarchive.xml";

        /// <summary>
        /// The name of the test file with a mix of sub references and local content.
        /// </summary>
        private const string TEST_HYBRIDFILENAME = @"..\..\test_files\testhybridarchive.xml";

        /// <summary>
        /// Changed files are saved under this name.
        /// </summary>
        private const string SAVE_FILENAME = SAVE_PATH + @"\savefile.xml";

        /// <summary>
        /// This is the path to a directory where files are saved.
        /// </summary>
        private const string SAVE_PATH = @"..\..\test_files\savedir\";

        /// <summary>
        /// Sets up the testing fixture.
        /// </summary>
        [SetUp]
        public void Setup()
        {
            if (!Directory.Exists(SAVE_PATH))
            {
                Directory.CreateDirectory(SAVE_PATH);
            }

            testObject1 = new XmlDocumentIndexSource();
            testObject1.Load(new SimpleIndexParameters(TEST_FILENAME));

            testObject2 = new XmlDocumentIndexSource();
        }

        /// <summary>
        /// Counts the elements reported by an enumerator.
        /// </summary>
        /// <param name="enumerator">The enumerator.</param>
        /// <returns>The number of elements.</returns>
        private static int CountEnumerator(IEnumerator enumerator)
        {
            int count = 0;
            while (enumerator.MoveNext())
            {
                count++;
            }

            return count;
        }

        /// <summary>
        /// Tests the <see cref="XmlDocumentIndexSource.IndexInfo"/> implementation. This property is a semi-constant
        /// getter-only. It's used to be able to reinstantiate an index, through reflection.
        /// </summary>
        [Test]
        public void TestIndexInfo()
        {
            IndexInfo info = testObject1.IndexInfo;

            Assert.AreEqual(type.Assembly.FullName, info.AssemblyName, "Assembly name mismatch.");
            Assert.AreEqual(type.FullName, info.ClassName, "Class name mismatch.");

            SimpleIndexParameters parameters = (SimpleIndexParameters) info.Parameters;
            Assert.AreEqual(TEST_FILENAME, parameters.FileName, "The test file name doesn't match.");
        }

        /// <summary>
        /// Asks for a document index based on the <see cref="IndexInfo"/> returned.
        /// </summary>
        [Test]
        public void TestIndexInfoForReal()
        {
            DocumentIndex docIndex = new IndexFactory().GetIndex(testObject1.IndexInfo);
            Assert.IsFalse(docIndex.Contains("ENTRU"), "ENTRU should not be present.");
            Assert.IsTrue(docIndex.Contains("entr"), "entr should be present.");
            Assert.IsTrue(docIndex.Contains("entre"), "entre should be present (equal to entr).");
        }

        /// <summary>
        /// If there is no file name, an exception should be thrown if we ask for the index info.
        /// </summary>
        [Test]
        [ExpectedException(typeof(InvalidOperationException))]
        public void TestIndexInfoNotSupported()
        {
            IndexInfo info = testObject2.IndexInfo;
        }

        /// <summary>
        /// Tests the <see cref="XmlDocumentIndexSource.Culture"/> implementation. This property has both a getter and
        /// setter. The default value is verified, and then the value is changed.
        /// </summary>
        [Test]
        public void TestCulture()
        {
            Assert.AreEqual(new CultureInfo("en-us"), testObject2.Culture, "Default value not returned.");
            testObject2.Culture = new CultureInfo("fr-FR");

            Assert.AreEqual(new CultureInfo("fr-FR"), testObject2.Culture, "Changed value not returned.");
            Assert.AreEqual(new CultureInfo("fr-FR"), testObject1.Culture, "File value not returned.");
        }

        /// <summary>
        /// Tests setting a null culture. This is, naturally, not allowed. An exception is thrown.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestCultureNull()
        {
            testObject1.Culture = null;
        }

        /// <summary>
        /// Tests changing culture for a non-empty index. This is not allowed.
        /// </summary>
        [Test]
        [ExpectedException(typeof(InvalidOperationException))]
        public void TestCultureNonEmpty()
        {
            testObject1.Culture = CultureInfo.InvariantCulture;
        }

        /// <summary>
        /// Tests the <see cref="XmlDocumentIndexSource.CompareOptions"/> implementation. This property has both a
        /// getter and setter. The default value is verified, and then the value is changed.
        /// </summary>
        [Test]
        public void TestCompareOptions()
        {
            Assert.AreEqual(CompareOptions.None, testObject2.CompareOptions, "Default value not returned.");
            testObject2.CompareOptions = CompareOptions.IgnoreNonSpace;

            Assert.AreEqual(CompareOptions.IgnoreNonSpace, testObject2.CompareOptions, "Changed value not returned.");
            Assert.AreEqual(CompareOptions.IgnoreNonSpace, testObject1.CompareOptions, "File value not returned.");
        }

        /// <summary>
        /// Tests changing compare options for a non-empty index. This is not allowed.
        /// </summary>
        [Test]
        [ExpectedException(typeof(InvalidOperationException))]
        public void TestCompareOptionsNonEmpty()
        {
            testObject1.CompareOptions = CompareOptions.IgnoreCase;
        }

        /// <summary>
        /// Tests the <see cref="XmlDocumentIndexSource.FilePath"/> implementation. This property has both a
        /// getter and setter. The constructor-set value is verified, and then the value is changed.
        /// </summary>
        [Test]
        public void TestFilePath()
        {
            Assert.IsNull(testObject2.FilePath, "Null expected for never initialized path.");
            Assert.AreEqual(TEST_FILENAME, testObject1.FilePath, "Normal file path not returned correctly.");

            testObject2.FilePath = "newfilename";
            Assert.AreEqual("newfilename", testObject2.FilePath, "Changed file path not returned correctly.");
        }

        /// <summary>
        /// Tests setting a null file path. This is only allowed as the initial value, never to be set again.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestFilePathNull()
        {
            testObject1.FilePath = null;
        }

        /// <summary>
        /// Tests setting an empty file path. An exception is thrown.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestFilePathEmpty()
        {
            testObject1.FilePath = " ";
        }

        /// <summary>
        /// Tests setting an invalid file path, being a directory.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestFilePathDirectory()
        {
            testObject1.FilePath = @"../../test_files";
        }

        /// <summary>
        /// Reads the number of sub files, and changes it to other, valid, values for the file-based test object.
        /// (Only 0 is allowed for the empty object.)
        /// </summary>
        [Test]
        public void TestNumberOfSubFiles()
        {
            Assert.AreEqual(2, testObject1.NumberOfSubFiles, "Number of subfiles incorrect for object 1.");
            Assert.AreEqual(0, testObject2.NumberOfSubFiles, "Number of subfiles incorrect for object 2.");

            testObject1.NumberOfSubFiles = 1;
            Assert.AreEqual(2, testObject1.NumberOfSubFiles, "Number of subfiles not changed for object 1.");
        }

        /// <summary>
        /// Tests a negative number of subfiles. This is never allowed.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentOutOfRangeException))]
        public void TestNumberOfSubFilesNegative()
        {
            testObject2.NumberOfSubFiles = -1;
        }

        /// <summary>
        /// Tests a number of subfiles higher than the real word count. This is not allowed.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentOutOfRangeException))]
        public void TestNumberOfSubFilesTooPositive()
        {
            testObject1.NumberOfSubFiles = 15;
        }

        /// <summary>
        /// We check that we can successfully lock on the <see cref="XmlDocumentIndexSource.SyncRoot"/>, although
        /// that may not be the complete story.
        /// </summary>
        [Test]
        public void TestSyncRoot()
        {
            lock (testObject1.SyncRoot)
            {
            }
        }

        /// <summary>
        /// Tests constructing with a null filename.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestFileNameConstructorNull()
        {
            new XmlDocumentIndexSource(null);
        }

        /// <summary>
        /// Tests constructing with an empty filename.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestFileNameConstructorEmpty()
        {
            new XmlDocumentIndexSource(" ");
        }

        /// <summary>
        /// Tests constructing with an invalid filename.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestFileNameConstructorInvalid()
        {
            new XmlDocumentIndexSource("\0");
        }

        /// <summary>
        /// Tests the filename constructor.
        /// </summary>
        [Test]
        public void TestFileNameConstructor()
        {
            Assert.AreEqual("somefile.xml", new XmlDocumentIndexSource("somefile.xml").FilePath,
                "Reported path not as expected.");
        }

        /// <summary>
        /// Tests adding words, both already existing ones (with different representations) and new ones.
        /// </summary>
        [Test]
        public void TestAdd()
        {
            WordInfo word = new WordInfo("entr", 999);
            testObject1.Add(word);

            // Diacritic-insensitive comparison
            IList list = testObject1["entre"];
            Assert.AreEqual(3, list.Count, "List count not as expected. 1");
            Assert.AreEqual(1, list[0], "First position not as expected. 1");
            Assert.AreEqual(3, list[1], "Second position not as expected. 1");
            Assert.AreEqual(999, list[2], "Third position not as expected. 1");

            testObject2.Add(word);
            // Diacritic-SENSITIVE comparison
            Assert.IsNull(testObject2["entre"], "Entre shouldn't match entr. 2");

            list = testObject2["entr"];
            Assert.AreEqual(1, list.Count, "Exactly one element expected. 2");
            Assert.AreEqual(999, list[0], "Position not as expected. 2");
        }

        /// <summary>
        /// Tests adding a null word. Exception expected.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestAddNull()
        {
            testObject1.Add(null);
        }

        /// <summary>
        /// Tests adding an empty word. Exception expected. The definition of an empty string is different in the
        /// document indexer and this component.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestAddEmpty()
        {
            testObject1.Add(new WordInfo(" ", 0));
        }

        /// <summary>
        /// Tests removing a word, both one which really exists and one which does not.
        /// </summary>
        [Test]
        public void TestRemove()
        {
            testObject1.Remove("entre");
            Assert.IsNull(testObject1["entr"], "Entr should now be a null list.");

            Assert.AreEqual(1, CountEnumerator(testObject1.GetEnumerator()), "Word count not as expected. 1");
            Assert.AreEqual(2, testObject1.NumberOfSubFiles,
                "The number of sub files is allowed to be higher than the number of words after removal.");

            // The count should remain zero, but no exception shold be thrown
            testObject2.Remove("entre");
            Assert.IsNull(testObject2["entre"], "Entre should now be a null list.");
            Assert.AreEqual(0, CountEnumerator(testObject2.GetEnumerator()), "Word count not as expected. 2");
        }

        /// <summary>
        /// Tests removing a null word. This should raise an exception.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestRemoveNull()
        {
            testObject1.Remove(null);
        }

        /// <summary>
        /// Tests removing an empty word. This should raise an exception.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestRemoveEmpty()
        {
            testObject1.Remove(" ");
        }

        /// <summary>
        /// A simple test of <see cref="XmlDocumentIndexSource.Contains"/>.
        /// </summary>
        [Test]
        public void TestContains()
        {
            Assert.IsTrue(testObject1.Contains("entre"), "Entr should exist.");
            Assert.IsFalse(testObject2.Contains("entre"), "Entre should not exist.");
        }

        /// <summary>
        /// Another test of <see cref="XmlDocumentIndexSource.Contains"/>. An empty list means that the method should
        /// return false.
        /// </summary>
        [Test]
        public void TestContainsEmptyList()
        {
            testObject1["entr"] = new int[0];
            Assert.IsFalse(testObject1.Contains("entre"), "Entr should not exist, when the list is empty.");
        }

        /// <summary>
        /// Tests checking for a null word. This should raise an exception.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestContainsNull()
        {
            testObject1.Contains(null);
        }

        /// <summary>
        /// Tests checking for an empty word. This should raise an exception.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestContainsEmpty()
        {
            testObject1.Contains(" ");
        }

        /// <summary>
        /// Tests clearing the store.
        /// </summary>
        [Test]
        public void TestClear()
        {
            testObject1.Clear();
            Assert.IsNull(testObject1["entr"], "Known word should now be null.");
            Assert.AreEqual(0, CountEnumerator(testObject1.GetEnumerator()), "Count should be zero after clearing.");
        }

        /// <summary>
        /// Tests <see cref="XmlDocumentIndexSource.GetEnumerator"/>, by iterating in order.
        /// </summary>
        [Test]
        public void TestGetEnumerator()
        {
            IDictionaryEnumerator enumerator = testObject1.GetEnumerator();
            for (int i = 0; i < 2; i++)
            {
                enumerator.MoveNext();
                Assert.AreEqual("entr", enumerator.Key, "The first key doesn't match.");
                Assert.AreEqual(2, ((IList) enumerator.Value).Count, "Wrong count of first word.");

                enumerator.MoveNext();
                Assert.AreEqual("rouge", enumerator.Key, "The second key doesn't match.");
                Assert.AreEqual(1, ((IList) enumerator.Value).Count, "Wrong count of second word.");

                Assert.IsFalse(enumerator.MoveNext(), "Moving next should fail after end.");
                enumerator.Reset();
            }
        }

        /// <summary>
        /// Tests loading, both for the already loaded file, and the other one.
        /// </summary>
        [Test]
        public void TestLoadSubFiles()
        {
            testObject1.Load(new SimpleIndexParameters(TEST_FILENAME));
            // This test verifies that the data is correct quite thoroughly
            TestGetEnumerator();

            testObject1 = testObject2;
            // TestGetEnumerator is written for testObject1, so let's just move them around
            testObject1.Load(new SimpleIndexParameters(TEST_FILENAME));
            // This test verifies that the data is correct quite thoroughly
            TestGetEnumerator();
        }

        /// <summary>
        /// Tests loading, but now with a monolithic file (without subfile references).
        /// </summary>
        [Test]
        public void TestLoadMainFile()
        {
            testObject1.Load(new SimpleIndexParameters(TEST_MAINFILENAME));
            // This test verifies that the data is correct quite thoroughly
            TestGetEnumerator();

            testObject1 = testObject2;
            // TestGetEnumerator is written for testObject1, so let's just move them around
            testObject1.Load(new SimpleIndexParameters(TEST_MAINFILENAME));
            // This test verifies that the data is correct quite thoroughly
            TestGetEnumerator();
        }

        /// <summary>
        /// Tests loading, but now with a hybrid file (both subfiles and word list in the main file).
        /// </summary>
        [Test]
        public void TestLoadHybridFile()
        {
            testObject1.Load(new SimpleIndexParameters(TEST_HYBRIDFILENAME));
            // This test verifies that the data is correct quite thoroughly
            TestGetEnumerator();

            testObject1 = testObject2;
            // TestGetEnumerator is written for testObject1, so let's just move them around
            testObject1.Load(new SimpleIndexParameters(TEST_HYBRIDFILENAME));
            // This test verifies that the data is correct quite thoroughly
            TestGetEnumerator();
        }

        /// <summary>
        /// Tests loading with a null reference. Exception is thrown.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestLoadNull()
        {
            testObject1.Load(null);
        }

        /// <summary>
        /// Tests loading with an empty file name. Exception is thrown.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestLoadEmpty()
        {
            testObject1.Load(new SimpleIndexParameters(" "));
        }

        /// <summary>
        /// Tests loading with a parameter object of incorrect type.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestLoadWrongType()
        {
            // this implements IParameters
            testObject1.Load(this);
        }

        /// <summary>
        /// If there is no file name set, it's impossible to save.
        /// </summary>
        [Test]
        [ExpectedException(typeof(InvalidOperationException))]
        public void TestSaveInvalid()
        {
            testObject2.Save();
        }

        /// <summary>
        /// Tests saving over and over again, with subfiles, in a subdirectory. The data is read back, and a
        /// verification is also done to make sure that the total file count in the directory won't increase.
        /// </summary>
        [Test]
        public void TestSaveWithSubfiles()
        {
            XmlDocumentIndexSource testObject = testObject1;
            testObject.FilePath = SAVE_FILENAME;
            for (int i = 0; i < 100; i++)
            {
                testObject.Save();

                // Save and load the data in another instance
                testObject1 = new XmlDocumentIndexSource();
                testObject1.Load(new SimpleIndexParameters(SAVE_FILENAME));
                TestGetEnumerator();
            }

            Assert.AreEqual(3, Directory.GetFiles(SAVE_PATH).Length, "Only three files should still be left.");
        }

        /// <summary>
        /// Tests saving over and over again, but now in single-file mode. The data is read back and verified.
        /// </summary>
        [Test]
        public void TestSaveMainFile()
        {
            XmlDocumentIndexSource testObject = testObject2;
            testObject2.Load(new SimpleIndexParameters(TEST_MAINFILENAME));

            testObject.FilePath = SAVE_FILENAME;
            for (int i = 0; i < 100; i++)
            {
                testObject.Save();

                // Save and load the data in another instance
                testObject1 = new XmlDocumentIndexSource();
                testObject1.Load(new SimpleIndexParameters(SAVE_FILENAME));
                TestGetEnumerator();
            }

            Assert.AreEqual(1, Directory.GetFiles(SAVE_PATH).Length, "Only one file should still be left.");
        }

        /// <summary>
        /// If we clear the index, the number of sub files decreases, but only when actually saving. This test verifies
        /// that behavior.
        /// </summary>
        [Test]
        public void TestSaveShrink()
        {

            testObject1.Clear();
            testObject1.FilePath = SAVE_FILENAME;
            Assert.AreEqual(2, testObject1.NumberOfSubFiles, "Number of sub files not as expected.");
            testObject1.Save();
            Assert.AreEqual(0, testObject1.NumberOfSubFiles, "Number of sub files should decrease when saving.");

            testObject2.Load(new SimpleIndexParameters(SAVE_FILENAME));
            Assert.AreEqual(0, testObject2.NumberOfSubFiles,
                "Number of sub files should be zero when loading again.");
        }

        /// <summary>
        /// Some tests for comparison, all with valid string parameters. Non-string arguments, resulting in exceptions,
        /// are tested in separate cases.
        /// </summary>
        [Test]
        public void TestCompare()
        {
            Assert.AreEqual(0, testObject1.Compare("entre", "entre"), "Identical strings are identical. 1");
            Assert.AreEqual(0, testObject1.Compare("entr", "entre"), "Identity depends on comparison options. 1");
            Assert.IsTrue(testObject1.Compare("B", "A") > 0, "B > A 1");
            Assert.IsTrue(testObject1.Compare("A", "B") < 0, "A < B 1");

            Assert.AreEqual(0, testObject2.Compare("entre", "entre"), "Identical strings are identical. 2");
            Assert.IsTrue(testObject2.Compare("entr", "entre") > 0, "Identity depends on comparison options. 2");
            Assert.IsTrue(testObject2.Compare("B", "A") > 0, "B > A 2");
            Assert.IsTrue(testObject2.Compare("A", "B") < 0, "A < B 2");
        }

        /// <summary>
        /// Parameters being null, compared against strings. Null is less than everything, except null.
        /// </summary>
        [Test]
        public void TestCompareNull()
        {
            Assert.IsTrue(testObject2.Compare(null, "") < 0, "Null should compare as less than empty string. 1");
            Assert.IsTrue(testObject2.Compare("", null) > 0, "Null should compare as less than empty string. 2");

            Assert.IsTrue(testObject2.Compare(null, null) == 0, "Null is equal to null.");
        }

        /// <summary>
        /// A null value compared against a non-string raises an exception.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestCompareNullAgainstNonString1()
        {
            testObject2.Compare(null, this);
        }

        /// <summary>
        /// A null value compared against a non-string raises an exception.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestCompareNullAgainstNonString2()
        {
            testObject2.Compare(this, null);
        }

        /// <summary>
        /// The getter part of the indexer is quite simple. One of the more important elements is to check that the
        /// returned value is only a clone, and nothing else.
        /// </summary>
        [Test]
        public void TestGetIndexer()
        {
            for (int i = 0; i < 2; i++)
            {
                IList list = testObject1["entre"];
                Assert.AreEqual(2, list.Count, "Two elements should be present.");
                Assert.AreEqual(1, list[0], "First position not as expected.");
                Assert.AreEqual(3, list[1], "Second position not as expected.");

                // Change the first entry, loop again, verifying that the stored data do not change
                list[0] = 99999;
            }

            Assert.IsNull(testObject1["I DON'T EXIST"],
                "Null should be returned for those words that are not present.");
        }

        /// <summary>
        /// Tests getting the list for a null word.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestGetIndexerNull()
        {
            IList list = testObject1[null];
        }

        /// <summary>
        /// Tests getting the list for an empty word.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestGetIndexerEmpty()
        {
            IList list = testObject1[" "];
        }

        /// <summary>
        /// Tests setting lists with the indexer, including that the type is always changed to <see cref="long"/>.
        /// </summary>
        [Test]
        public void TestSetIndexer()
        {
            testObject1["someword"] = new int[] {2, 3};
            IList list = testObject1["someword"];

            Assert.AreEqual(2, list[0], "The first value not as expected.");
            Assert.AreEqual(3, list[1], "The second value not as expected.");

            for (int i = 0; i < 2; i++)
            {
                Assert.AreEqual(typeof(Int64), list[i].GetType(), "Element type not as expected.");
            }
        }

        /// <summary>
        /// Tests setting the list for a null word. An exception is thrown.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestSetIndexerNullWord()
        {
            testObject1[null] = new long[0];
        }

        /// <summary>
        /// Tests setting the list for an empty word. An exception is thrown.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestSetIndexerEmptyWord()
        {
            testObject1[" "] = new long[0];
        }

        /// <summary>
        /// Tests setting a null list. An exception is thrown.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestSetIndexerNullList()
        {
            testObject1["someword"] = null;
        }

        /// <summary>
        /// Some tests save files for testing. Remove those files.
        /// </summary>
        [TearDown]
        public void Teardown()
        {
            foreach (string file in Directory.GetFiles(SAVE_PATH))
            {
                File.Delete(file);
            }
        }
    }
}
